
#include "aconfig.h"
#include "nntpd.h"
#include "aprotos.h"

ACL	*acls[MAX_ACLS];

PROTO int loadaccess(void) {
    FILE *inf;
    char ip[16], pfx[3], acs[64], profile[MAX_NAME];
    char buf[128];
    struct in_addr inaddr;
    ulong inaddrt;
    ulong ipaddrs;
    int linenr=0;
    char *p;
    struct stat _stat;

    if ( (inf = fopen(cfg.AccessFile,"r")) == NULL ) {
	die("Can't open %s", cfg.AccessFile);
	return 1;
    }

    while( fgets(buf,sizeof(buf),inf) ) {
	linenr++;
        if ( buf[0] != '\r' && buf[0] != '\n' && buf[0] != ';' && buf[0] != '#' ) {
	    profile[0] = '\0';

	    /*
	     * Read and store ip ranges for access control
	     */
	    if ( sscanf(buf, 
			      "%15[0-9.]%*[/]%2[0-9] %64s%*[ \t]%32s", 
			      ip, pfx, acs, profile 
			      ) >= 3 ) {
		inet_aton(ip, &inaddr);
		inaddrt = ntohl(inaddr.s_addr);
		if ( atol(pfx) == 0 )
		    ipaddrs = 0xFFFFFFFF;
		else
		    ipaddrs = (1 << (32-atol(pfx)))-1;

		if ( (acls[master->numacls] = calloc(1,sizeof(ACL)) ) == NULL ) 
			die("Can't allocate memory for ACL");

		if ( ! nullstr(profile) && getprofile(profile) == NULL ) 
		    die("Undefined profile on line %d of %s", linenr, cfg.AccessFile);
		acls[master->numacls]->profile = nullstr(profile) ? getprofile("default") : getprofile(profile);

		p = strtok(acs,",");
		do {
		    if ( !strcasecmp(p,"read") ) 
			acls[master->numacls]->read = 1;
		    else 
		    if ( ! strcasecmp(p,"stats") ) 
			acls[master->numacls]->stats = 1;
		    else 
		    if ( ! strcasecmp(p,"post") ) 
			acls[master->numacls]->post = 1;
		    else 
		    if ( ! strcasecmp(p,"apost") ) 
			acls[master->numacls]->apost = 1;
		    else 
		    if ( ! strcasecmp(p,"deny") )
			acls[master->numacls]->deny = 1;
		    else 
		    if ( ! strcasecmp(p,"auth") )
			acls[master->numacls]->auth = 1;
		    else 
		    if ( ! strcasecmp(p,"unlimit") )
			acls[master->numacls]->unlimit = 1;
		    else {
			die("Wrong ACL entry \"%s\" on line %d of %s", p, linenr, cfg.AccessFile);
			return 3;
		    }
		}while( (p=strtok(NULL,",")) );

		acls[master->numacls]->start = inaddrt;
		acls[master->numacls]->end   = inaddrt + ipaddrs;
		acls[master->numacls]->size  = ipaddrs;

		master->numacls++;
		if ( master->numacls > MAX_ACLS ) 
		    die("Too many access entries (%d) increase MAX_ACLS in nntpd.h");

	    }else{
		die("ACL syntax error on line %d of %s", linenr, cfg.AccessFile);
		fclose(inf);
		return 3;
	    }
	}
        pfx[0] = 0;
        ip[0] = 0;
    }

    fclose(inf);
    stat(cfg.AccessFile,&_stat);
    master->laststataccess = _stat.st_mtime;

    info("Loaded %d ACLs",    master->numacls);
    return 0;
}

PROTO void freeaccess(void) {
    int i;

    for (i=0; i<master->numacls; i++) 
	free(acls[i]);
    info("Freed %d ACLs",master->numacls);
    master->numacls = 0;
}

PROTO int reloadaccess(void) {
    struct stat _stat;

    if ( stat(cfg.AccessFile,&_stat) == -1 ) 
	die("Can't stat access file %s", cfg.AccessFile);

    if ( master->laststataccess < _stat.st_mtime ) {
	freeaccess();
	return loadaccess();
    }
    return 0;
}


PROTO ACL* getacl(ulong ipaddr) {
    int i;

    for (i=0; i<master->numacls; i++) {
	if ( ipaddr >= acls[i]->start && ipaddr <= acls[i]->end )
	    return acls[i];
    }
    return NULL;
}

