/*
 * auth.c - Authentication and Accounting Configuration
 *
 * $Id: auth.c,v 1.6 2004/02/26 19:26:31 tommy Exp $
 */

#include "aconfig.h"
#include "nntpd.h"
#include "memory.h"
#include "aprotos.h"

/* define this if you don't want shared libs to be closed on exit
 * this can cause sig[segv|ill]'s on exit */
#undef DONT_CLOSE_LIBS
                                                                                                                         
AUTH *auths;
char *auth_default = "auth_default";
char *acct_default = "acct_default";
void *auth_handler;
void *acct_handler;

PROTO char* (*dl_check_auth)(CLIENT *, CONFIG *, char *);
PROTO void (*dl_acct_entry)(CLIENT *, CONFIG *, char *);

/*
 * Configuration Functions
 */

PROTO void reloadauth(void) {
    struct stat _stat;

    if ( stat(cfg.AuthFile, &_stat) == -1 ) 
	die("Can't stat auth file %s", cfg.AuthFile);

    if ( master->laststatauth < _stat.st_mtime ) {
	clearauth();
	loadauth();
    }
}

PROTO void loadauth(void) {
    FILE *f;
    struct stat _stat;
    char buf[1024], b1[MAX_STRING], b2[MAX_NAME], b3[MAX_NAME], b4[MAX_NAME];
    int nauths=0;
    int linenr=0;

    if ( (auths=memmap(sizeof(AUTH) * MAX_AUTHS)) == NULL ) {
	die("Cant mmap auths");
    }

    if ( (f=fopen(cfg.AuthFile,"r")) == NULL )
	die("Can't open auth config file %s", cfg.AuthFile);

    while( fgets(buf,sizeof(buf),f) ) {
	chop(buf);
	linenr++;
	if ( buf[0] != '\r' && buf[0] != '\n' && buf[0] != ';' && buf[0] != '#' && buf[0] != 0 ) {

	    b1[0] = 0; b2[0] = 0; b3[0] = 0; b4[0] = 0;

	    if ( sscanf(buf, "%255s %31s %31s %31s", b1, b2, b3, b4) > 0 ) {
		if ( ( nullstr(b2) == 0 && getalias(b2) == NULL ) ||
		     ( nullstr(b3) == 0 && getalias(b3) == NULL ) )
		    die("Undefined Alias on line %d of %s", linenr, cfg.AuthFile);

		if ( nullstr(b4) == 0 && getprofile(b4) == NULL ) 
		    die("Undefined Profile on line %d of %s", linenr, cfg.AuthFile);

		(auths+nauths)->mask    = strdup(b1);
		(auths+nauths)->auth    = nullstr(b2) == 1 ? getalias(auth_default) : getalias(b2);
		(auths+nauths)->acct    = nullstr(b3) == 1 ? getalias(acct_default) : getalias(b3);
		(auths+nauths)->profile = nullstr(b4) == 1 ? getprofile("default") : getprofile(b4);
		nauths++;
	    }else{
		die("Parse error on line %d of %s", linenr, cfg.AuthFile);
	    }
	}
    }

    fclose(f);
    master->numauths = nauths;
    info("Loaded %d authenticators",nauths);

    stat(cfg.AuthFile,&_stat);
    master->laststatauth = _stat.st_mtime;
}

PROTO void clearauth(void) {
    memunmap(auths,   sizeof(AUTH)  * MAX_AUTHS);
    info("Freed %d Authenticators", master->numauths);
    master->numauths = 0;
    return;
}

PROTO AUTH* getauth(const char *username) {
    int i;

    for (i=0; i<master->numauths; i++)
	if ( uwildmat_simple(username, (auths+i)->mask) )
	    return (auths+i);

    return NULL;
}

/*
 * Loadable Module Operations
 */

PROTO int load_auth_module(void) {
    char l[512];
    sprintf(l, "%s/lib%s", cfg.ModulePath, client->auth->auth->lib);

    auth_handler = dlopen(l, RTLD_LAZY);
    if ( ! auth_handler ) {
        syslog(LOG_ERR,"cant load auth_handler module %s", dlerror());
        return -1;
    }
                                                                                                                         
    dl_check_auth = dlsym(auth_handler,"check_auth");
    return 0;
}
                                                                                                                         
PROTO int load_acct_module(void) {
    char l[512];
    sprintf(l, "%s/lib%s", cfg.ModulePath, client->auth->acct->lib);

    acct_handler = dlopen(l, RTLD_LAZY);
    if ( ! acct_handler ) {
        syslog(LOG_ERR,"cant load acct_handler module %s", dlerror());
        return -1;
    }
                                                                                                                         
    dl_acct_entry = dlsym(acct_handler,"acct_entry");
    return 0;
}

PROTO int unload_auth_module(void) {
#ifndef DONT_CLOSE_LIBS
    dlclose(auth_handler);
#endif
    return 0;
}
                                                                                                                         
PROTO int unload_acct_module(void) {
#ifndef DONT_CLOSE_LIBS
    dlclose(acct_handler);
#endif
    return 0;
}

