#include <stdio.h>
#include <string.h>
#include <syslog.h>

#include "aconfig.h"
#include "nntpd.h"
#include "aprotos.h"

#define _MAX_LINEBUF 256


PROTO char* handshake_pop3(int sock, char *buf, int size, int timeout) {
    int n;
 
    if ( (n=read_socket(sock, buf, size, timeout)) <= 0 )
        return("Connection Failed: %m");
 
    buf[n] = 0;
 
    if ( buf[0] != '+' && buf[1] != 'O' && buf[2] != 'K' ) 
        return buf;
 
    return NULL;
}


const char * check_auth(CLIENT *client, CONFIG *cf, char *args) {
    char retbuf[_MAX_LINEBUF];
    char server[128];
    int port;
    int sock;
    char *err;

    if ( sscanf(args, "%128[^:]:%d", server, &port) != 2 ) {
	syslog(LOG_ERR,"Wrong argument syntax %s for auth_remote", args);
	return MSG_AUTH_ERR;
    }

    if ( (sock=connect_socket(server, port)) == -1 ) {
        syslog(LOG_ERR, "auth_pop3: Could not connect to server %s", server);
	return MSG_AUTH_ERR;
    }
 
    if ( (err=handshake_pop3(sock, client->sbuf, cf->BufSize, cf->ServerReadTimeout)) != NULL ) {
        syslog(LOG_ERR, "auth_pop3: Server %s failed: %s", server, err);
	return MSG_AUTH_ERR;
    }

    client->serversock = sock;
    client->connected=1;
    strncpy(client->lastserver, server, MAX_HOSTNAME-1);

    writeserver(client,"USER %s", client->username);
    if ( readserver(client, retbuf, _MAX_LINEBUF) == NULL ) return MSG_AUTH_ERR;
    writeserver(client,"PASS %s", client->password);
    if ( readserver(client, retbuf, _MAX_LINEBUF) == NULL ) return MSG_AUTH_ERR;

    if ( strncasecmp("+OK", retbuf, 3) == 0 ) {
	client->authenticated++;
	send_quit(client);
	close(client->serversock);
	client->connected=0;

	return MSG_AUTH_OK;
    }
    return MSG_AUTH_REJ;
}
