#include "nntpd.h"

#define PROTO

MASTER *master = NULL;
CLIENT *client = NULL;


PROTO void die(const char *str, ...) {
    va_list ap;
    char s[1024];
     
    va_start(ap, str);
    vsprintf(s, str, ap);

    if ( errno ) {
	syslog(LOG_ERR, "died: %s (%s)", s, strerror(errno));
	fprintf(stderr, "died: %s (%s)\n", s, strerror(errno));
    }else{
	syslog(LOG_ERR, "died: %s", s);
	fprintf(stderr, "died: %s\n", s);
    }

    va_end(ap);
    exit(1);
}

PROTO void info(const char *str, ...) {
    va_list ap;
    char s[1024];
     
    va_start(ap, str);
    vsprintf(s, str, ap);

    syslog(LOG_NOTICE, "%s", s);
    fprintf(stderr, "%s\n", s);

    va_end(ap);
}


PROTO void closeall(int fd) {
    int fdlimit = sysconf(_SC_OPEN_MAX);
    while (fd < fdlimit)
	close(fd++);
}

PROTO char* strtolower(char *str) {
    int i=0;
    while(str[i]) { 
	str[i] = tolower(str[i]); 
	i++;
    }
    return str;
}

/*
 * add \r\n if not already present
 */
PROTO char* fixrn(char *str) {
    char p[1024];
    int ln = strlen(str);

    if ( str[ln-2] == '\r' && str[ln-1] == '\n' ){
        return strdup(str);

    }else if ( str[ln-2] == '\n' && str[ln-1] == '\r' ){
	strncpy(p, str, ln-2);
	p[ln-2] = '\0';
	return strdup(strcat(p, "\r\n"));

    }else if ( str[ln-1] == '\n' || str[ln-1] == '\r' ) {
	strncpy(p, str, ln-1);
	p[ln-1] = '\0';
	return strdup(strcat(p, "\r\n"));

    }else{
	strcpy(p, str);
	return strdup(strcat(p, "\r\n"));
    }
}

/*
 * strip \r\n combinations
 */
PROTO char* striprn(const char *s) {
    char *str, *end;
        
    str = strdup(s);
    end = str;

    str += strlen(end) - 1;
    while( *str == '\r' || *str == '\n' ) {
	*str = '\0';
	str--;
    }
    return end;
}
                                                                                                                                                           
PROTO int nullstr(const char *s) {
    if ( s == NULL ) return 1;
    if ( s[0] == '\0' ) return 1;
    return 0;
}


PROTO void chop(char *buf) {
    char *s;

    s = buf;
    s += strlen(buf) -1;
    while ( *s == '\r' || *s == '\n' || *s == ' ' ) {
	*s = '\0';
	s--;
    }
} 


/*
 * Strip "pattern" from buffer
 * tommy@news-service.com,*@news-service.com gets tommy
 * news-service@tommy,news-service@* gets tommy
 */

PROTO char* strippat(char *buf, char *pat) {
    char *s;
    int l = strlen(buf)-1;
    int j = strlen(pat)-1;

    if ( pat[0] == '*' ) {
	s = strdup(buf);

	while( pat[j] != '*' && l > 0 ) {
	    s[l] = 0;
	    j--;
	    l--;
	}
	return s;

    }else if ( pat[j] == '*' ) {
	s = strdup(buf);

	while( *pat++ != '*' ) {
	    *s++;
	}
	return s;
    }
    return NULL;
}


/*
 * checkbuf functions
 * check for \r\n.\r\n end of article in a buffer
 */

PROTO void checkbuf_init(char *buf) {
    buf[0] = '\0';
    buf[1] = '\0';
    buf[2] = '\0';
    buf[3] = '\r';
    buf[4] = '\n';
}

PROTO int checkbuf_isend(char *buf, char *rcvbuf, int bread) {
    char chkbuf[5];
    int j, k;

    if ( bread < 5 ) {
        k=0;

	/* compose the buf from our checkbuf and received data */
        for (j=0; j<5-bread; j++ )
            chkbuf[k++] = buf[j+bread];
        for (j=0; j<bread; j++)
            chkbuf[k++] = rcvbuf[j];

	/* check if it's the end.. */
	if ( chkbuf[0] == '\r' &&
	     chkbuf[1] == '\n' &&
	     chkbuf[2] == '.'  &&
	     chkbuf[3] == '\r' &&
	     chkbuf[4] == '\n' )
	    return 1;
    }else{
	/* check if perhaps this is the end already */
	if ( rcvbuf[bread-5] == '\r' &&
	     rcvbuf[bread-4] == '\n' &&
	     rcvbuf[bread-3] == '.'  &&
	     rcvbuf[bread-2] == '\r' && 
	     rcvbuf[bread-1] == '\n' )
	    return 1;

	/* copy the last 5 bytes of the receive buf to checkbuf */
        for (j=0; j<5; j++)
            buf[j] = rcvbuf[bread-5+j];

    }

    return 0;
}
