/*
 * nntpd.c  General client allocation and command parser routines
 * $Id: nntpd.c,v 1.49 2004/03/07 01:30:42 tommy Exp $
 */

#include "nntpd.h"
#include "aconfig.h"
#include "setproctitle.h"
#include "aprotos.h"

const char *VERSION = _VERSION;

#define CLEARPASS memset(&client->password, 0, MAX_USERPASS);

char *helptxt = "100 Supported Commands\r\n"
		"   MODE READER\r\n"
		"   AUTHINFO USER <username>\r\n"
		"   AUTHINFO PASS <password>\r\n"
		"   LIST <active|newsgroups|overview.fmt> <pattern>\r\n"
		"   XGTITLE <pattern>\r\n"
		"   GROUP <newsgroup>\r\n"
		"   LISTGROUP <newsgroup>\r\n"
		"   NEWGROUPS <yy><yymmdd> <hhmmss>\r\n"
		"   OVER <range|msgid>\r\n"
		"   XOVER <range|msgid>\r\n"
		"   XHDR <header> <range|msgid>\r\n"
		"   XPAT <header> <range|msgid> <pattern> <pattern..>\r\n"
		"   NEWNEWS <newsgroup> <yymmdd> <hhmmss> <gmt|utc>\r\n"
		"   STAT <msgid|number>\r\n"
		"   HEAD <msgid|number>\r\n"
		"   BODY <msgid|number>\r\n"
  		"   ARTICLE <msgid|number>\r\n"
		"   POST\r\n"
		"   NEXT\r\n"
		"   LAST\r\n"
		"   HELP\r\n"
		"   DATE\r\n"
		"   QUIT\r\n"
		"The NEWNEWS command is only partly supported for single newsgroups only.\r\n"
		"The ARTICLE command can only be used inside a newsgroup.\r\n"
		"\r\n"
		"Chiparus NNTPSwitch-%s (c) 2002\r\n"
		".\r\n";

CMDINFO commands[] = {	     /* auth, acct, limit */
    { "MODE",	    cmd_mode,	    1, 0, 0 },
    { "SLAVE",	    cmd_slave,	    0, 0, 0 }, /* not in help */
    { "AUTHINFO",   cmd_authinfo,   0, 0, 0 },
    { "LIST",	    cmd_list,	    1, 0, 0 },
    { "XGTITLE",    cmd_xgtitle,    1, 0, 0 },
    { "GROUP",	    cmd_group,	    1, 0, 0 },
    { "LISTGROUP",  cmd_listgroup,  1, 0, 0 },
    { "NEWGROUPS",  cmd_newgroups,  1, 0, 0 },
    { "OVER",	    cmd_over,	    1, 0, 0 },
    { "XOVER",	    cmd_xover,	    1, 0, 0 },
    { "XHDR",	    cmd_xhdr,	    1, 0, 0 },
    { "XPAT",	    cmd_xpat,	    1, 0, 0 },
    { "NEWNEWS",    cmd_newnews,    1, 0, 0 },
    { "STAT",	    cmd_stat,	    1, 0, 0 },
    { "HEAD",	    cmd_head,	    1, 1, 1 },
    { "BODY",	    cmd_body,	    1, 1, 1 },
    { "ARTICLE",    cmd_article,    1, 1, 1 },
    { "POST",	    cmd_post,	    1, 0, 0 },
    { "NEXT",	    cmd_next,	    1, 0, 0 },
    { "LAST",	    cmd_last,	    1, 0, 0 },
    { "HELP",	    cmd_help,	    0, 0, 0 },
    { "DATE",	    cmd_date,	    0, 0, 0 },
    { "QUIT",	    cmd_quit,	    0, 0, 0 },
    { "STATISTICS", cmd_statistics, 0, 0, 0 },
    { "IHAVE",	    cmd_unsupported,0, 0, 0 }, /* not in help */
    { "GET",	    cmd_http,	    0, 0, 0 }, 
    { NULL,	    cmd_unsupported,0, 0, 0 },
};

PROTO void client_error(CLIENT *client, char *txt) {
    syslog(LOG_NOTICE, "%s: client error %s ",client->hostname, txt);
    writeclient(client,MSG_GENERIC_ERROR,txt);
    close_child(client);
    exit(0);
}

PROTO void close_child(CLIENT *client) {
    syslog(LOG_NOTICE,"%s: connection closed articles %d bytes %lu groups %d posts %d postbytes %lu",
		client->hostname, client->articles, client->bytes, 
		client->groups, client->posts, client->postbytes);

    if ( client->authenticated ) {
	if ( ! load_acct_module() ) {
	    dl_acct_entry(client, &cfg, client->auth->acct->args);
	    unload_acct_module();
	}else{
	    syslog(LOG_DEBUG,"%s: accounting vanished", client->hostname);
	}
    }
    client->profile->connections--;
    free_client(client->id);
    master->connections--;
    return;
}

PROTO void close_child_noacc(CLIENT *client) {
    syslog(LOG_NOTICE,"%s: connection closed articles %d bytes %lu groups %d posts %d",
	client->hostname, client->articles, client->bytes, client->groups, client->posts);
    client->profile->connections--;
    free_client(client->id);
    master->connections--;
    return;
}

PROTO CLIENT* find_client(ulong ip4addr, int *concur, int maxcon) {
    int i;
    int cc=0;
    CLIENT *cl;

    /* check for both a free slot and the number of   
     * concurrent sessions at once to speed things up */

    for (i=0; i<MAX_PROC; i++) {
	if ( (master->clients+i)->ip4addr == ip4addr )
	    cc++;
	if ( (master->clients+i)->inuse == 0 ) {
	    cl = (master->clients+i);   // set client pointer to one from the list
	    memset(cl, '\0', sizeof(CLIENT));
	    cl->inuse++;
	    cl->id = i;
	    cl->ip4addr = ip4addr;

	    /* check for more concurrent connections */
	    for ( ; i<MAX_PROC; i++) {
		if ( (master->clients+i)->ip4addr == ip4addr )
		    cc++;
		if ( maxcon > 0 && cc > maxcon ) {
		    (*concur) = cc;
		    cl->inuse = 0;
		    cl->ip4addr = 0;
		    return NULL;
		}
	    }
	    (*concur) = cc;
	    cl->concur = cc;
	    cl->starttime = time(NULL);
	    return cl;
	}
    }
    info("Too many slots requested, increase MAX_PROC in nntpd.h");
    return NULL;
}

PROTO void free_client(int clientid) {
    memset(master->clients+clientid, '\0', sizeof(CLIENT));
}


/* this is called directly after forking a child and after client is allocated */
PROTO void run_child(CLIENT *cl, char *hostname, ulong ipadr)
{
    char *rc = NULL;
    client = cl;

    master->connections++;
    setproctitle("nntpswitchd %d: %s connecting", client->id, client->hostname);

    /* futher initialize client */
    client->hostname	    = strdup(hostname);
    client->ip4addr	    = ipadr;
    client->errstr	    = NULL;
    client->profile	    = client->acl->profile;
    client->profile->connections++;
    client->group	    = NULL;
    client->groupserver	    = NULL;

    if ( (client->bbuf  = malloc(cfg.BufSize+1)) == NULL ) 
	client_error(client,"No room for bbuf");
    
    if ( (client->sbuf  = malloc(cfg.BufSize+1)) == NULL ) 
	client_error(client,"No room for sbuf");

    openlog("nntpswitchd", LOG_PID, LOG_NEWS);
    syslog(LOG_NOTICE, "connect from %s: clientid %d clientnum %d total %d",
			client->hostname, client->id, client->concur, master->connections);

    if ( client->profile->connections > client->profile->MaxConnections ) {
	swriteclient(client, MSG_TOOMANY);
	close_child(client);
	return;
    }

    writeclient(client,"%d %s NNTPSwitch-%s, %d groups available, %s, slot %d, connections %d\r\n", 
			 client->acl->post ? 200 : 201,   client->profile->Banner, VERSION, master->numgroups,
			 client->acl->post ? "posting allowed" : "no posting",
			 client->id + 1, client->concur
			 );

    setproctitle("nntpswitchd %d: %s idle",client->id, client->hostname);

    /* main client loop */
    errno=0;
    while ( client->error == 0 && 
	    (rc=readclient(client,client->bbuf,cfg.BufSize,0)) != NULL && 
	    time(NULL) - client->starttime < client->profile->MaxSessionTime &&
	    client->bytes < client->profile->MaxSessionBytes ) {

	client->bbuf[strlen(client->bbuf)] = 0;
	if ( parsecmd(client, client->bbuf) )
	    break;
	setproctitle("nntpswitchd %d: %s idle",client->id, client->hostname);
    	errno=0;
    }

    /*
     * check what kind of errors we have or had en exit the client based on that
     * this code in general and way of errorhandling is worse than lame
     */

    if ( client->error > 0 ) {
        syslog(LOG_DEBUG,"%s: timeout or error %s", client->hostname, client->errstr);

    }else if ( client->bytes >= client->profile->MaxSessionBytes ) {
	syslog(LOG_DEBUG,"%s: session byte limit reached", client->hostname);
	swriteclient(client, MSG_BYTELIMIT);

    }else if ( time(NULL) - client->starttime >= client->profile->MaxSessionTime ) {
	syslog(LOG_DEBUG,"%s: session time limit reached %s", client->hostname, client->timeout ? "(timeout)" : "(normal)");
	swriteclient(client, MSG_SESSIONLIMIT);

    }else if ( rc == NULL && client->timeout == 1 ) {
        syslog(LOG_DEBUG,"%s: client timeout after %d seconds", client->hostname, cfg.ClientReadTimeout);
        writeclient(client,"205 Timeout after %d seconds, articles %d, bytes %ld, groups %d, posts %d\r\n", 
			   cfg.ClientReadTimeout, client->articles, client->bytes, client->groups, client->posts);

    }else if ( errno ) {
        syslog(LOG_DEBUG,"%s: timeout or error fixme %m", client->hostname);
        writeclient(client,"205 Timeout/Error articles %d, bytes %ld, groups %d, posts %d\r\n",client->articles, client->bytes, client->groups, client->posts);
    
    }else{
        syslog(LOG_DEBUG,"%s: eof from client", client->hostname);
    }

    client_quit(client);
    close_child(client);
    return;
}


/* 
 * return true to exit, false to continue accepting commands
 */
PROTO int parsecmd(CLIENT *client, char *buf) {
    char a0[512],a1[512];
    CMDINFO *cmd;
    char *err;
    int i;

    a0[0] = 0; a1[0] = 0;
    sscanf(buf, "%31[^\r\n\t ] %480[^\r\n]", a0, a1);
    if (a0[0] != '\0') {
        syslog(LOG_DEBUG, "%s: command %s %s",client->hostname, a0, a1);
        setproctitle("nntpswitchd %d: %s command %s %s",client->id, client->hostname,a0,a1);

        for( cmd = commands; cmd->command; cmd++ ) 
	    if ( !strcasecmp(a0,cmd->command) ) 
		break;

	client->command = cmd;
        if ( cmd->auth == 1 && client->acl->auth == 1 && client->authenticated == 0 ) {
	   return swriteclient(client, MSG_AUTHREQUIRED);
	}

	switch( cmd->num ) {

	    case cmd_quit:
		writeclient(client,"205 Exit articles %d, bytes %ld, groups %d, posts %d\r\n",
				    client->articles, client->bytes, client->groups, client->posts);
		client_quit(client);
		close_child(client);
		_exit(0);

	    case cmd_http:
		swriteclient(client, MSG_COMMAND_HTTP);
		client_quit(client);
		close_child(client);
		_exit(0);

	    case cmd_help:
		return writeclient(client, helptxt, VERSION);

	    case cmd_date:
		send_date(client);
		return 0;

	    case cmd_mode:
		if ( !strcasecmp(a1,"reader") ) {
		    if ( client->connected == 1 ) {
    			if ( ! writeserver(client,"MODE READER\r\n") ) return 1;
    			if ( (readserver(client, a1, MAX_SERVERRSP-1)) == NULL ) return 1;
    			if ( atoi(a1) > 299 ) {
			    syslog(LOG_ERR,"%s: mode reader check server %s response %s",client->hostname, client->lastserver, a1);
			    writeclient(client,a1);
			    return 1;
    			}
		    }
		    return writeclient(client,"%d Sure!\r\n", client->acl->post ? 200 : 201);
		}else
		    return swriteclient(client,"500 Unsupported MODE command\r\n");

	    case cmd_slave:
		return swriteclient(client,"202 Sure!\r\n");

	    case cmd_authinfo:
		sscanf(a1,"%16[^\r\n\t ] %64[^\r\n]",a0,buf);

		if ( !strcasecmp(a0,"user") ) {
		    strcpy(client->username, buf);
		}else if ( !strcasecmp(a0,"pass") ) {
		    strcpy(client->password, buf);
		}else{
		    return swriteclient(client,"500 Unsupported AUTHINFO command\r\n");
		}

		if ( client->username[0] == 0 || client->password[0] == 0 )
		    return swriteclient(client, MSG_MOREAUTHREQ);

		client->auth = getauth(client->username);
		if ( ! load_auth_module() ) {
		    err = strdup(dl_check_auth(client, &cfg, client->auth->auth->args));
		    unload_auth_module();
		    syslog(LOG_DEBUG,"authinfo result %s", err);
		    CLEARPASS
		    if ( client->authenticated && client->profile != client->auth->profile ) {
			if ( client->auth->profile->connections < client->auth->profile->MaxConnections ) {
			    client->profile->connections--;
			    client->profile = client->auth->profile;
			    client->profile->connections++;
			}else{
			    return swriteclient(client, MSG_AUTH_PROFILEFULL);
			}
		    }
		    return swriteclient(client,err);
		}else{
		    CLEARPASS
		    return swriteclient(client,MSG_AUTH_ERR);
		}
	   
	    case cmd_list:
		return cmdlist(client, a1);
	    
	    case cmd_xgtitle:
		return cmdxgtitle(client, a1);

	    case cmd_group:
		group(client, a1, false);
		return 0;
	   
	    case cmd_newgroups:
		newgroups(client, a1);
		return 0;

	    case cmd_listgroup:
		if ( strlen(a1) > 0 ) {	    
		    // groupname has been specified by user 
		    if ( group(client, a1, true) == 0 ) {
		        return passcmd(client, a0, a1, cmd);
		    }else{
			return swriteclient(client, MSG_NOSUCHGROUP);
		    }
		}else{
		    if ( client->group == NULL )
		        // no current group selected 
		        return swriteclient(client, MSG_NOGROUP);
		    else
		        // no group given, use current selected group 
		        return passcmd(client, a0, "", cmd); 
		}
		return 0;

	    case cmd_xover:
		if ( client->group == NULL ) 
		    return swriteclient(client, MSG_NOGROUP);
		else
		    return passcmd(client,a0, a1, cmd);

	    case cmd_xpat:
	    case cmd_xhdr:
		if ( client->group == NULL ) {
		    return swriteclient(client, MSG_NOGROUP);
		}else{
		    if ( cfg.DenyNoIndexField ) {
			sscanf(a1,"%128[^\r\n\t ]",buf);

			if ( isinoverviewfmt(buf) ) {
			    return passcmd(client,a0, a1, cmd);
			}else{
			    return swriteclient(client, MSG_NOPATINDEX);
			}

		    }else{
			return passcmd(client,a0, a1, cmd);
		    }

		}

	    case cmd_over:
		if ( client->group == NULL ) 
		    return swriteclient(client, MSG_NOGROUP);
		else
		    return passcmd(client, "XOVER", a1, cmd);

	    case cmd_head:
		if ( client->group == NULL ) 
		    return swriteclient(client, MSG_ARTICLECANT);
		else
		    return passcmd(client,a0, a1, cmd);
		return 0;

	    case cmd_article:
	    case cmd_body:
		if ( client->group == NULL ) 
		    return swriteclient(client, MSG_ARTICLECANT);
		else {
		    client->articles++;
		    client->serverarts++;
		    client->grouparts++;
		    return passcmd(client,a0, a1, cmd);
		}
		return 0;

	    case cmd_newnews:
		sscanf(a1,"%256[^\r\n\t ] %256[^\r\n]",a0,buf);
		if ( !strcmp(a0,"newnews") || strchr(a0,'*') != NULL || strchr(a0,'!') != NULL ) {
		    return swriteclient(client, MSG_NONEWNEWS);
		}else{
		    if ( group(client, a0, true) == 0 ) {
		        return passcmd(client, "NEWNEWS", a1, cmd);
		    }else{
			return swriteclient(client, MSG_NOSUCHGROUP);
		    }
		}
		return 0;

	    case cmd_stat:
	    case cmd_last:
	    case cmd_next:
		if ( client->group == NULL ) 
		    return swriteclient(client, MSG_NOGROUP);
		else {
		    if ( ! writeserver(client,"%s %s\r\n",a0,a1) ) break;
		    if ( (readserver(client, buf, 256)) == NULL ) {
			client_error(client, UMSG_SERVERTIMEOUT);
		    }else{
			return swriteclient(client,buf);
		    }
		}
		return 0;

	    case cmd_post:
		if ( client->acl->post == 0 )
		    return swriteclient(client, MSG_NOPOSTING);
		else
		    return post(client);

	    case cmd_unsupported:
		return swriteclient(client, MSG_UNSUPPORTED);

	    case cmd_statistics:
		if ( client->acl->stats != 1 ) 
		    return swriteclient(client, MSG_UNSUPPORTED);

		for (i=0; i<MAX_PROC; i++) {
		    writeclient(client, "100 %d %d %d.%d.%d.%d %s %s\r\n", 
				    i,
				    (master->clients+i)->inuse,
				    ((master->clients+i)->ip4addr & 0xff000000) / (256*256*256),
				    ((master->clients+i)->ip4addr & 0xff0000) / (256*256),
				    ((master->clients+i)->ip4addr & 0xff00) / 256,
				    (master->clients+i)->ip4addr & 0xff,
				    (master->clients+i)->group->newsgroup, 
				    (master->clients+i)->username);
		}
		return 0;

	    default:
		return swriteclient(client, MSG_NOTIMPLEMENTED);
	}
    }
    return 0;
}


