#include <unistd.h>
#include <sys/time.h>

#include "aconfig.h"
#include "nntpd.h"
#include "aprotos.h"

#define BUF_SIZE 32768

#define CLEARBUF buffer[0]='\0';

struct _hlist {
    char *hdr;
    char *val;
    struct _hlist *next;
};
#define hlist struct _hlist


/* wait for client to finish sending data */
void wait_finish_client(CLIENT *client, char *buffer, int bufsize) {
    int i=1;

    while( client->error == 0 && i == 1 &&
		    readclient(client, buffer, bufsize, 0) != NULL )
    { 
	if ( buffer[0] == '.' && (buffer[1] == '\r' || buffer[1] == '\n') )
	    i=0;
    }
}

void free_hlist(hlist *hl) {
    if ( hl->next == NULL ) {
	free(hl);
    }else{
	free_hlist(hl->next);
    }
}

hlist *insert_hlist(hlist *first, char *hdr, char *val) {
    hlist *i = first, *last = NULL;

    while( i != NULL ) { last=i; i=i->next; }
    
    i = malloc(sizeof(hlist));
    if ( i == NULL ) { die("No room for post header"); }
    if ( last != NULL ) last->next = i;
    i->hdr = strdup(hdr);
    i->val = strdup(val);
    i->next = NULL;
    return i;
}


int mysystem (char *command) {
    struct sigaction sigchld;

    /* gotta ignore children because we're a child ourself */ 
    memset(&sigchld,0,sizeof(sigchld));
    sigchld.sa_handler = SIG_IGN;
    sigaction(SIGCHLD, &sigchld, NULL);

    return system(command);
}


/*
 * walk through newsgroup lists and set emailaddress
 * of first moderated newsgroups. return groupname
 * if the group is unknown, NULL if group is valid
 */
char * check_moderated(char *groups, char **mailto) {
    char *p;
    char *g;
    ACTIVE *grp;

    p=strtok(groups, ",");
    do {
	if ( (grp = getgroup(p)) == NULL )
	    return p;

	if ( grp != NULL && grp->mode == 'm' ) {
	    for (g=p; *g!='\0'; g++) 
		if ( *g == '.' ) *g = '-';

	    *mailto = malloc(strlen(p) + strlen(cfg.ModeratorDomain) + 2);
	    if ( *mailto == NULL ) die("No room for moderatordomain");

	    strcpy(*mailto, p);
	    strcat(*mailto, "@");
	    strcat(*mailto, cfg.ModeratorDomain);

	    return NULL;
	}
    }while( (p=strtok(NULL,",")) );
    return p;
}


/*
 * Send the email to moderator address.
 */
int send_email(CLIENT *client, hlist *phlist, char *msgid, char *postbuf, int postsize, char *address) {
    char tmp[19] = "/tmp/nntpsw.XXXXXX";
    int fd;
    char buf[MAX_HEADER + 512];

    if ( (fd=mkstemp(tmp)) == -1 ) die("Cant write spool file for moderator");

    /* spool the headers and body first */

    while( phlist != NULL ) {
        if ( phlist->hdr == NULL && phlist->val == NULL ) {
	    phlist=phlist->next;
	    continue;
	}

        sprintf(buf, "%s %s\r\n", phlist->hdr, phlist->val);
        if ( (write(fd, buf, strlen(buf))) == -1 ) {
	    writeclient(client, "500 Spooling for moderator failed (headers)");
	    return 1;
	}
	phlist = phlist->next;
    }

    sprintf(buf, "To: %s\r\n\r\n", address);

    if ( (write(fd, buf, strlen(buf))) == -1 ) {
	writeclient(client, "500 Spooling for moderator failed (separator)");
	return 1;
    }

    if ( ! write(fd, postbuf, postsize) ) {
	writeclient(client, "500 Spooling for moderator failed (body)");
        disconnect_server(client);
	return 1;
    }

    close(fd);

    /* finally, mail the spooled post */

    sprintf(buf, "%s < %s", cfg.MailerPath, tmp);

    if ( mysystem(buf) ) {
    	unlink(tmp);
	return writeclient(client, MSG_POST_NOMAIL);
    }

    unlink(tmp);
    return writeclient(client, MSG_POST_MAILED);
}


/*
 * connect to the post server and send the message using ihave command
 * the message is a header list and raw data of the body.
 * return 1 on error 
 */
int send_ihave(CLIENT *client, hlist *phlist, char *msgid, char *postbuf, int postsize) {
    char b[256];
    ACTIVE *oldgrp;

    oldgrp = client->group;

    if ( client->connected == 1 ) disconnect_server(client);
    if ( connect_server_nntp(client, cfg.PostServer, cfg.PostServerPort, cfg.ServerReadTimeout) ) {
	writeclient(client, MSG_SERVER_DOWN);
	return 0;
    }

    if ( ! writeserver(client,"IHAVE %s\r\n", msgid) ) return 1;
    if (  (readserver(client, b, 255)) == NULL ) return 1;
    if ( atoi(b) > 399 ) {
	syslog(LOG_ERR,"%s: post server responded %s",client->hostname, b);
	writeclient(client,b);
        disconnect_server(client);
	return 0;
    }else{
	syslog(LOG_NOTICE,"%s: post server responded %s",client->hostname, b);
    }

    /* post the headers */

    while( phlist != NULL ) {
        if ( phlist->hdr == NULL && phlist->val == NULL ) {
	    phlist=phlist->next;
	    syslog(LOG_DEBUG, "skipping empty header?");
	    continue;
	}

        if ( ! writeserver(client, "%s %s", phlist->hdr, phlist->val) ) return 1;
	phlist = phlist->next;
    }

    /* post the body */
    
    if ( ! writeserver(client,"\r\n") ) return 1;
    if ( ! write(client->serversock, postbuf, postsize) ) {
        syslog(LOG_ERR,"cant write to server %m");
        disconnect_server(client);
	return 1;
    }

    if ( (readserver(client, b, 255)) == NULL ) return 1;
    if ( atoi(b) > 299 ) {
	syslog(LOG_ERR,"%s: post server responded %s",client->hostname, b);
	writeclient(client,b);
        disconnect_server(client);
	return 0;
    }
    
    writeclient(client, MSG_POST_RECEIVED);

    client->serverarts++;
    client->serverbytes += strlen(postbuf);
    disconnect_server(client);

    /* so now we can restore the old group with this lame workaround */
    /* actually, if we just could have a 2nd server connection this  */
    /* would have been unneeded. so, FIXME */
    if ( oldgrp != NULL ) 
        group(client, oldgrp->newsgroup, true);

    return 0;
}


PROTO int post(CLIENT *client) {
    int i=1;
    int gotnewsgroups=0, gotfrom=0, gotsubject=0, gotmsgid=0, gotdate=0, gotorg=0, gotlines=0, isapproved=0;
    int gotcontrol=0;
    char *postbuf = NULL;
    int postsize=0, s;
    char b[256];
    struct timeval tv;
    struct tm *ptm;
    char msgid[64], realmsgid[128], timestr[64];
    char hedr[64], val[MAX_HEADER];
    char newsgroups[MAX_HEADER];
    char *mailto = NULL;
    char chkbuf[5];
    char *p;
    hlist *hdr = NULL;
    char *buffer;

    if ( (buffer=malloc(BUF_SIZE)) == NULL ) 
	die("no room for post buffer");

    writeclient(client, MSG_POST_OK);

    /* make Path the first header */
    hdr = insert_hlist(hdr, "Path:", client->profile->PathHeader);

    /* 
     * Read headers from the client and check if we have everything 
     * we need. Also skip system headers if we find them.
     */

    while( i && (readclient(client, buffer, BUF_SIZE, 0)) != NULL ) {
	chop(buffer);

	if ( buffer[0] == 0 ) {
	    i=0; 
	    break; 
	}

	if ( buffer[0] == '.' && buffer[1] == 0 )
	    return swriteclient(client, MSG_POST_CANCEL);

	if ( strlen(buffer) >= MAX_HEADER ) {
	    wait_finish_client(client, buffer, BUF_SIZE);
	    return swriteclient(client, MSG_POST_HDRTOOLONG);
	}

	hedr[0] = 0;
	val[0] = 0;
	if ( (sscanf(buffer,"%63[^\t ] %1023[^\r\n]", hedr, val)) == 0 ) {
	    /* we have a multi-line header */
	    strncpy(val, buffer, MAX_HEADER-1);
	}

	/* skip empty headers */
	if ( val[0] == '\0' ) 
	    continue;

	/* headers to skip from client */
	if ( !strncasecmp(hedr, "NNTP-Posting-Host:",  18) ) continue;
	if ( !strncasecmp(hedr, "NNTP-Posting-Date:",  18) ) continue;
	if ( !strncasecmp(hedr, "Complaints-To:",      14) ) continue;
	if ( !strncasecmp(hedr, "X-Complaints-To:",    16) ) continue;
	if ( !strncasecmp(hedr, "X-Trace:",		8) ) continue;
	if ( !strncasecmp(hedr, "Path:", 		5) ) continue;
	if ( !strncasecmp(hedr, "To:", 		 	3) ) continue;
	if ( !strncasecmp(hedr, "Cc:", 		 	3) ) continue;
	if ( !strncasecmp(hedr, "Bcc:", 		4) ) continue;

	/* if no permission to post approved posts, skip header */
	if ( !strncasecmp(hedr, "Approved:", 9 ) ) {
	    if ( client->acl->apost == 1 ) {
		isapproved++;
	    }else{
		continue;
	    }
	}

	/* if forced organization overwrite, just skip it here */
	if ( client->profile->ForceOrganization && !strncasecmp(hedr, "Organization:", 13) ) 
	    continue;

	/* check and store the newsgroups line */
	if ( !strncasecmp(hedr, "Newsgroups:",   11) ) {
	    gotnewsgroups++;
	    strncpy(newsgroups, val, MAX_HEADER-1);
	}

	/* check some required headers */
	if ( !strncasecmp(hedr, "From:",          5) ) gotfrom++;
	if ( !strncasecmp(hedr, "Subject:",       8) ) gotsubject++;
	if ( !strncasecmp(hedr, "Date:", 	  5) ) gotdate++;
	if ( !strncasecmp(hedr, "Organization:", 13) ) gotorg++;
	if ( !strncasecmp(hedr, "Lines:", 	  6) ) gotlines++;
	if ( !strncasecmp(hedr, "Control:", 	  8) ) gotcontrol++;
	if ( !strncasecmp(hedr, "Message-ID:",   11) ) {
	    gotmsgid++;
	    strcpy(realmsgid, val);
	}

	insert_hlist(hdr, hedr, striprn(val));
    }

    /*
     * Continue reading the body from the client
     */

    i=1;
    checkbuf_init(chkbuf);

    while( i == 1 && client->error == 0 && client->timeout == 0 && 
		    (s=read_socket(client->socket, buffer, BUF_SIZE, cfg.ClientReadTimeout)) > 0 ) 
    {
	if ( checkbuf_isend(chkbuf, buffer, s) )
	    i=0;

	postbuf = (char*)realloc(postbuf, postsize+s);
	if ( postbuf == NULL ) die("no room for postbuffer + s");
	memcpy(postbuf+postsize, buffer, s);
	postsize += s;
    }

    /*
     * Now we should have all data from the client.
     */

    if ( postbuf == NULL || i == 1 ) {
	syslog(LOG_ERR,"%s: Post Aborted or No body received",client->hostname);
	free_hlist(hdr);
	return 1;
    }

    /* check required headers */
    if ( gotnewsgroups == 0 || gotfrom == 0 || gotsubject == 0 ) {
        free(postbuf);
	free_hlist(hdr);
	return swriteclient(client, MSG_POST_WRONGHEADER);
    }
    
    /* check group permission */
    if ( ! match_expression(newsgroups, getwildmat(client->profile->PostPat), 0) ) {
	free(postbuf);
	free_hlist(hdr);
	return swriteclient(client, MSG_NOPOSTING);
    }

    /* check permission to send control messages */
    if ( client->profile->NoControl == 1 && gotcontrol == 1 ) {
	free(postbuf);
	free_hlist(hdr);
	return swriteclient(client, MSG_NOPOSTING);
    }

    /* create and insert message-id */
    gettimeofday(&tv, NULL);
    sprintf(msgid,"%.5lx%lx%lx%.5d", tv.tv_usec, tv.tv_sec, client->ip4addr^0xA5A5A5A5L, (int)getpid());

    if ( ! gotmsgid ) {
        sprintf(realmsgid,"<%lx$%lx$%lx$%d@%s>", 
			tv.tv_usec, 
			tv.tv_sec, 
			client->ip4addr, 
			(int)getpid(), 
			client->profile->MessageIdDomain );
        insert_hlist(hdr, "Message-ID:", realmsgid);
    }

    /* fixup system headers */

    insert_hlist(hdr, "X-Complaints-To:", client->profile->XComplaintsTo);
    if ( ! gotorg )  {
        sprintf(b,"%s\r\n", client->profile->Organization);
        insert_hlist(hdr, "Organization:", client->profile->Organization);
    }

    ptm = localtime(&tv.tv_sec);
    strftime(timestr, sizeof(timestr), "%a, %d %b %Y %H:%M:%S %z",ptm);

    if ( ! gotdate )  {
	insert_hlist(hdr, "Date:", timestr);
    }

    if ( ! gotlines )  {
	for( i=0; i<postsize-3; i++ )
	    if ( postbuf[i] == '\r' && postbuf[i+1] == '\n' ) gotlines++;
	sprintf(hedr, "%d", gotlines);
	insert_hlist(hdr, "Lines:", hedr);
    }

    /* log and insert trace headers */

    if ( client->profile->AddNNTPPostingHost ) {
        insert_hlist(hdr, "NNTP-Posting-Host:", client->hostname);
	insert_hlist(hdr, "NNTP-Posting-Date:", timestr);
    }

    insert_hlist(hdr, "X-Trace:", msgid);
    if ( client->authenticated == 1 ) {
	syslog(LOG_NOTICE,"X-Trace: %s %s@%s %s\r\n", msgid, client->username, client->hostname, realmsgid);
    } else {
	syslog(LOG_NOTICE,"X-Trace: %s %s %s\r\n", msgid, client->hostname, realmsgid);
    }

    if ( (p=check_moderated(newsgroups, &mailto)) != NULL ) {
        syslog(LOG_NOTICE,"%s: Post to nonexistent group %s", client->hostname, p);
        free(postbuf);
	free_hlist(hdr);
	return writeclient(client, MSG_NOSUCHGROUP);
    }

    if ( isapproved == 0 && mailto != NULL )
	i = send_email(client, hdr, realmsgid, postbuf, postsize, mailto);
    else
        i = send_ihave(client, hdr, realmsgid, postbuf, postsize);

    free(postbuf);
    free_hlist(hdr);
    client->posts++;
    client->postbytes+=postsize;

    return i;
}

