
#include "aconfig.h"
#include "nntpd.h"
#include "aprotos.h"

#define CLEARBUF client->bbuf[0] = 0;

PROTO void client_quit(CLIENT *client) {
    if ( client->connected == 1 ) {
	closegroup(client);
	disconnect_server(client);
    }
}

PROTO int send_date(CLIENT *client) {
    struct timeval tv;
    struct tm *ptm;
    char timestr[40];

    gettimeofday(&tv,NULL);
    ptm = gmtime(&tv.tv_sec);
    strftime(timestr, sizeof(timestr), "%Y%m%d%H%M%S",ptm);

    /* FIXME: This ain't no UTC! */
    return writeclient(client,"111 %s UTC\r\n",timestr);
}


PROTO int passcmd(CLIENT *client, char *command, char *args, CMDINFO *cmd) {
    int     i = 1;
    char    chkbuf[5];
    ssize_t bread = 0;

    if ( ! writeserver(client, "%s %s\r\n", command, args) ) return -1;

    if ( (readserverline(client->bbuf, cfg.BufSize)) == NULL ) return -1;
    if ( swriteclient(client, client->bbuf) ) return -1;
    if ( atoi(client->bbuf) > 399 ) i=0;

    checkbuf_init(chkbuf);
    while (i && ! client->error) {
	if ( (bread = nreadserver(client, client->bbuf, cfg.BufSize)) == 0 )
	    return -1;

	if ( checkbuf_isend(chkbuf, client->bbuf, bread) ) 
	    i = 0;

	if ( slwriteclient(client, client->bbuf, bread, cmd->limit) ) 
	    return -1;

	if ( cmd->acct == 1 ) 
	    client->bytes += (ulong)bread;

	client->groupbytes  += (ulong)bread;
    }
    return 0;
}

PROTO void closegroup(CLIENT *client) {
    if ( client->group!= NULL && (client->grouparts > 0 || client->groupbytes > 0) ) {
    	syslog(LOG_NOTICE, "%s: group %s closed articles %d bytes %lu", 
				client->hostname, 
				client->group->newsgroup, 
				client->grouparts, 
				client->groupbytes);
    	client->grouparts = 0;
    	client->groupbytes = 0;
    }
}

PROTO int group(CLIENT *client, const char *group, char nogroupecho) {
    char b[MAX_SERVERRSP];
    ACTIVE *newgroup;
    SERVER *svr, *primary;
    int has_changed;
    CLEARBUF

    if ( nullstr(group) ) {
	swriteclient(client,"500 You must select a group\r\n");
	return 1;
    }
   
    has_changed = 1;
    if ( client->group != NULL && strcmp(client->group->newsgroup, group) == 0 )
        has_changed = 0;

    /* check for read permission */
    if ( has_changed == 1 && ! match_expression(group, getwildmat(client->profile->ReadPat), 0) ) {
	if ( ! nogroupecho )
	    swriteclient(client, MSG_NOSUCHGROUP);
        return 1;
    }

    /* check if we have the group at all and load group properties */
    if ( has_changed == 0 ) {
	newgroup = client->group;
    }else{
        if ( (newgroup = getgroup(group)) == NULL ) {
	    if ( ! nogroupecho ) 
	        swriteclient(client, MSG_NOSUCHGROUP);
	    return 1;
	}
    }

    if ( (svr=getserver(newgroup->server)) == NULL )
	return swriteclient(client, "500 Cant find server for group\r\n");

    /* disconnect first if we have to change server */
    if ( client->groupserver != NULL && 
	 client->connected == 1 &&
	 strcmp(svr->Groups, client->groupserver->Groups) != 0 ) {

        disconnect_server(client);
    }

    /* only close group if group has changed */
    if ( has_changed ) 
    	closegroup(client);

    client->group = newgroup;

    if ( client->connected != 1 ) {
	/* now get the real server and take balancing stuff in account */
	if ( (svr=selectserver_primary(newgroup->server)) == NULL )
	    return swriteclient(client, "500 Cant find server for group\r\n");

	primary = svr;	/* remember where we started with */
	client->groupserver = svr;

	if ( connect_server_nntp(client, svr->Hostname, svr->Port, svr->Timeout) ) {
	    if ( errno ) {
	        syslog(LOG_ERR,"%s: remote server %s down %m", client->hostname, client->groupserver->Name);
		errno = 0;
	    }

	    /* On server down find backup servers and try to conect to them */
	    while( client->connected != 1 && (svr = selectserver_backup(client->groupserver, primary)) != NULL ) {
    		client->groupserver = svr;
		if ( connect_server_nntp(client, svr->Hostname, svr->Port, svr->Timeout) ) {
	    	    if ( errno ) {
	                syslog(LOG_ERR,"%s: backup server %s down %m", client->hostname, client->groupserver->Name);
		        errno = 0;
	    	    }
		}
	    }
	
	    if ( client->connected != 1 ) {
		sleep(cfg.DownDelay);
	        writeclient(client, MSG_SERVER_DOWN);
	        client->group = NULL;
	        client->groupserver = NULL;
	        return 1;
	    }
	}
    }

    if ( ! writeserver(client, "GROUP %s\r\n", group) ) return -1;
    if ( ! readserver(client, b, MAX_SERVERRSP-1) ) return -1;

    if ( has_changed ) 
        client->groups++;

    syslog(LOG_NOTICE, "%s: group %s", client->hostname, group);
    if ( ! nogroupecho ) return swriteclient(client,b);
    return 0;
}

PROTO int cmdlist(CLIENT *client, char *args) {
    char cmd[16];

    if ( strlen(args) == 0 )
    	return listactive(client,"*");

    CLEARBUF
    sscanf(args,"%16[^\r\n\t ] %256[^\r\n]", cmd, client->bbuf);
    if ( client->bbuf[0] == 0 ) strcpy(client->bbuf,"*");

    if ( !strncasecmp(cmd,"active.times",12) ) {
	listactivetimes(client);

    }else if ( !strncasecmp(cmd,"active",6) ) {
    	listactive(client,client->bbuf);

    }else if ( !strncasecmp(cmd,"newsgroups",10)) {
    	listnewsgroups(client, client->bbuf);

    }else if ( !strncasecmp(cmd,"overview",8)) {
    	listoverviewfmt(client);

    }else{
        return swriteclient(client,"500 Unsupported LIST command\r\n");
    }
    return 0;
}

PROTO int cmdxgtitle(CLIENT *client, char *args) {
    if ( strlen(args) == 0 )
    	return listxgtitle(client,"*");

    CLEARBUF
    sscanf(args,"%256[^\r\n]", client->bbuf);
    if ( client->bbuf[0] == 0 ) strcpy(client->bbuf,"*");

    listxgtitle(client, client->bbuf);
    return 0;
}

