/*
 * NNTPSwitch Server Configuration
 */
#include "aconfig.h"
#include "nntpd.h"
#include "memory.h"
#include "aprotos.h"

int linenr;
SERVER *servers;


int cmp_server_level(const void *a, const void *b) {
                                                                                                                             
    const SERVER *s1 = *((const SERVER **) a);
    const SERVER *s2 = *((const SERVER **) b);
                                                                                                                             
    if ( s1->Level > s2->Level )
        return -1;
                                                                                                                             
    if ( s1->Level < s2->Level )
        return 1;
                                                                                                                             
    return 0;
}
                                                                                                                             

PROTO void clear_servers(void) {
    memunmap(servers, sizeof(SERVER) * MAX_SERVERS);
    info("Freed %d Servers", master->numservers);
    master->numservers = 0;
}


PROTO void reload_servers(void) {
    struct stat _stat;

    if ( stat(cfg.ServerFile, &_stat) == -1 ) 
	die("Can't stat server config file %s", cfg.ServerFile);

    if ( master->laststatserver < _stat.st_mtime ) {
	clear_servers();
	load_servers();
    }
}


PROTO void load_servers(void) {
    FILE *f;
    struct stat _stat;
    char buf[MAX_CFGLINE];
    SERVER *def;
    int i,s;
    
    if ( (servers=memmap(sizeof(SERVER) * MAX_SERVERS)) == NULL )
	die("Cant mmap servers");
    memset(servers, 0, sizeof(SERVER) * MAX_SERVERS);
    
    if ( (f=fopen(cfg.ServerFile,"r")) == NULL )
	die("Can't open server config %s", cfg.ServerFile);

    linenr=0;
    while( fgets(buf, MAX_CFGLINE, f) ) {
	chop(buf);
	linenr++;

	if ( buf[0] == ';' || buf[0] == '#' || buf[0] == 0 )
	    continue;

	if ( strncasecmp(buf, "server", 6) == 0 ) 
	    parsecfg_server(buf,f);
	else
	    die("Syntax error in %s on line %d", cfg.ServerFile, linenr);
    } 
    fclose(f);
    
    if ( getserver("default") == NULL )
	die("Server \"default\" not defined");

    /* Sort the servers by level which we'll use later */
    def = getserver("default");
    s=0;
    for (i=0; i<master->numservers; i++ )
        if ( (servers+i) != def )
            master->lservers[s++] = (servers+i);
                                                                                                                             
    qsort(master->lservers, master->numservers-1, sizeof(SERVER*), cmp_server_level);
                                                                                                                             
    info("Loaded %d Servers", master->numservers);
    
    stat(cfg.ServerFile,&_stat);
    master->laststatserver = _stat.st_mtime;
}


PROTO void parsecfg_server(char *buf, FILE *f) {
    char key[64], val[MAX_CFGLINE];
    int i;
    SERVER *def;

    if ( (sscanf(buf, "%s %s", key, val)) != 2 )
	die("Server Syntax error on line %d of %s", linenr, cfg.ServerFile);

    if ( strcmp(val, "default") != 0 ) {
	/* we inherit (copy) from default server */
	if ( (def = getserver("default")) == NULL ) 
	    die("Could not retrieve default server, the default server must be listed first.");

	memcpy((servers+master->numservers), def, sizeof(SERVER));
    }
    strcpy((servers+master->numservers)->Name, val);

    i=0;
    while( !i && fgets(buf, MAX_CFGLINE, f) ) {
	chop(buf);

	linenr++;
	if ( buf[0] == ';' || buf[0] == '#' || buf[0] == 0 )
	    continue;

	if ( strncasecmp(buf, "end", 3) == 0 ) {
	    i++;
	    break;
	}

	if ( sscanf(buf, "%*[ \t]%64s%*[ \t]%16384s[^\r\n]%*[\t\r\n]", key, val) != 2 )
	    die("Error on line %d of %s (%s - %s)", linenr, cfg.ServerFile, key, val);

	if ( strcasecmp(key, "hostname") == 0 ) {
	    strcpy((servers+master->numservers)->Hostname, val);
	}else if ( strcasecmp(key, "groups") == 0 ) {
	    strcpy((servers+master->numservers)->Groups, val);
	}else if ( strcasecmp(key, "timeout") == 0 ) {
	    (servers+master->numservers)->Timeout = atoi(val);
	}else if ( strcasecmp(key, "port") == 0 ) {
	    (servers+master->numservers)->Port = atoi(val);
	}else if ( strcasecmp(key, "level") == 0 ) {
	    (servers+master->numservers)->Level = atoi(val);
	    if ( (servers+master->numservers)->Level > MAX_LEVEL ) 
		die("Level out of range at line %d of %s", linenr, cfg.ServerFile);
	}else if ( strcasecmp(key, "type") == 0 ) {
	    if ( strcasecmp(val, "spool") == 0 ) {
		(servers+master->numservers)->ServerType = type_spool;
	    }else if ( strcasecmp(val, "xover") == 0 ) {
		(servers+master->numservers)->ServerType = type_xover;
	    }else if ( strcasecmp(val, "post") == 0 ) {
		(servers+master->numservers)->ServerType = type_post;
	    }else{
		die("Unknown server type on line %d of %s", linenr, cfg.ServerFile);
	    }
	}else if ( strcasecmp(key, "policy") == 0 ) {
	    if ( strcasecmp(val, "single") == 0 ) {
		(servers+master->numservers)->Policy = policy_single;
	    }else if ( strcasecmp(val, "backup") == 0 ) {
		(servers+master->numservers)->Policy = policy_backup;
	    }else if ( strcasecmp(val, "balance") == 0 ) {
		(servers+master->numservers)->Policy = policy_balance;
	    }else{
		die("Unknown server policy on line %d of %s", linenr, cfg.ServerFile);
	    }
	}else if ( strcasecmp(key, "activetimes") == 0 ) {
	    if ( strcasecmp(val, "false") == 0 ) {
		(servers+master->numservers)->ActiveTimes = 0;
	    }else if ( strcasecmp(val, "true") == 0 ) {
		(servers+master->numservers)->ActiveTimes = 1;
	    }else{
		(servers+master->numservers)->ActiveTimes = atoi(val);
	    }
	}else if ( strcasecmp(key, "descriptions") == 0 ) {
	    if ( strcasecmp(val, "false") == 0 ) {
		(servers+master->numservers)->Descriptions = 0;
	    }else if ( strcasecmp(val, "true") == 0 ) {
		(servers+master->numservers)->Descriptions = 1;
	    }else{
		(servers+master->numservers)->Descriptions = atoi(val);
	    }
	}else if ( strcasecmp(key, "splitlist") == 0 ) {
	    if ( strcasecmp(val, "false") == 0 ) {
		(servers+master->numservers)->SplitList = 0;
	    }else if ( strcasecmp(val, "true") == 0 ) {
		(servers+master->numservers)->SplitList = 1;
	    }else{
		(servers+master->numservers)->SplitList = atoi(val);
	    }
	}else{
	    info("parseserver unknown %s = %s", key, val);
	}
    }

    master->numservers++;
}


PROTO SERVER* getserver(const char *name) {
    int i;
                                                                                                                             
    for (i=0; i<master->numservers; i++)
        if ( strcmp((servers+i)->Name, name) == 0 )
	    return (servers+i);

    return NULL;
}

/* count the number of servers with the same level as us */
int count_server_level(int level) {
    int i;
    int num=0;

    for (i=0; i<master->numservers; i++) 
	if ( (servers+i)->Level == level )
	    num++;

    return num;
}


PROTO SERVER* selectserver_primary(const char *name) {
    int i;
    int num;
    SERVER *s;
    
    s = getserver(name);

    /* single or backup policy's just pick the first server */
    if ( s == NULL || s->Policy == policy_single || s->Policy == policy_backup ) 
	return s;

    /* otherwise we have to pick one according to the balancer */

    num = count_server_level(s->Level);
 
    /* roll the number */
    master->balance_id[s->Level]++;
    if ( master->balance_id[s->Level] >= num )
	master->balance_id[s->Level] = 0;

    /* finally find and select the server */
    num = master->balance_id[s->Level];
    for (i=0; i<master->numservers-1; i++) 
	if ( master->lservers[i]->Level == s->Level ) {
	    if ( num == 0 )
		return master->lservers[i];
	    num--;
	}

    syslog(LOG_DEBUG, "Hmm, could not find the other balancing server for %s", s->Name);
    return NULL;
}

/*
 * Find the next server with the same level. This most likely
 * is a backup servers. Give backup servers the same level
 * in the servers.conf. 
 */
PROTO SERVER* selectserver_backup(SERVER *current, SERVER *primary) {
    int i;
    int currnum = 0;

    /* dont check single server policies */
    if ( current == NULL || current->Policy == policy_single ) return NULL;

    /* get number of current server */
    for (i=0; i<master->numservers; i++) 
	if ( servers+i == current ) {
	    currnum = i;
	    break;
	}

    /* Start searching upwards in the list */
    for (i=currnum-1; i>0; i--) {
	if ( (servers+i) == primary )
	    return NULL;
	if ( (servers+i)->Level == current->Level && (servers+i) != current )
	    return (servers+i);
    }

    /* Hmm nothing. Then search from the bottom till the current */
    for (i=master->numservers-1; i>currnum; i--) {

	if ( (servers+i) == primary )
	    return NULL;
	if ( (servers+i)->Level == current->Level && (servers+i) != current )
	    return (servers+i);
    }

    /* too bad, no backup server */
    syslog(LOG_DEBUG, "No next server for policy in %s", current->Name);
    return NULL;
}
