/*
 * socket.c  Lowlevel socket server/client routines
 * $Id: socket.c,v 1.28 2004/02/23 23:22:41 tommy Exp $
 */

#include "aconfig.h"
#include "nntpd.h"
#include "aprotos.h"

#define CLEARBUFS client->sbuf[0]='\0';

PROTO void set_errormsg(char *str, ...) {
    va_list ap;

    if ( client->error > 0 ) {
	/* we probably had a broken pipe somewhere by now.. */
        syslog(LOG_NOTICE,"set_errormsg called twice %s", str);
        return;
    }

    if ( (client->errstr = calloc(1,512)) == NULL )
        die("No memory for errstr");

    client->error++;
    va_start(ap, str);
    vsprintf(client->errstr, str, ap);
    va_end(ap);
}


PROTO ssize_t read_socket(int socket, char *buf, int size, int timeout) {
    fd_set rdfs;
    struct timeval tv;

    FD_ZERO(&rdfs);
    FD_SET(socket, &rdfs);

    tv.tv_sec  = timeout;
    tv.tv_usec = 0;

    if ( select(socket+1, &rdfs, NULL, NULL, &tv) == 0 ) {
        syslog(LOG_DEBUG, "read_socket: select timeout");
	return -1;
    }

    return read(socket, buf, size);
}

PROTO ssize_t write_socket(int socket, char *buf, int size, int timeout) {
    fd_set rdfs;
    struct timeval tv;

    FD_ZERO(&rdfs);
    FD_SET(socket, &rdfs);

    tv.tv_sec  = timeout;
    tv.tv_usec = 0;

    if ( select(socket+1, NULL, &rdfs, NULL, &tv) == 0 ) {
        syslog(LOG_DEBUG, "write_socket: select timeout");
	return -1;
    }

    return write(socket, buf, size);
}


/* 
 * readserver() read a single line from the currently selected server
 */
PROTO char * readserver(CLIENT *client, char *buf, int size) {
    fd_set rdfs;
    struct timeval tv;
    size_t bread;

    FD_ZERO(&rdfs);
    FD_SET(client->serversock, &rdfs);

    tv.tv_sec  = cfg.ServerReadTimeout;
    tv.tv_usec = 0;

    if ( select(client->serversock+1, &rdfs, NULL, NULL, &tv) == 0 ) {
        set_errormsg("remote server %s read timeout", client->lastserver);
	/* FIXME: Somethimes this has to be written to the client, sometimes not
	 * writeclient(client,"500 Remote server read timeout");
	 */
        client->connected = 0;
	return NULL;
    }

    bread = read(client->serversock, buf, size);
    if ( bread == -1 ) {
	set_errormsg("cant read from server %s %m", client->lastserver);
	bread=0;
    }
    buf[bread] = 0;
    if ( cfg.LogReadserver )
        syslog(LOG_DEBUG,"readserver: %s", buf);
    client->serverbytes += (ulong)bread;
    return buf;
}

/* 
 * nreadserver() same as readserver but return number of bytes read (or 0)
 */
PROTO int nreadserver(CLIENT *client, char *buf, int size) {
    fd_set rdfs;
    struct timeval tv;
    size_t bread;

    FD_ZERO(&rdfs);
    FD_SET(client->serversock, &rdfs);

    tv.tv_sec  = cfg.ServerReadTimeout;
    tv.tv_usec = 0;

    if ( select(client->serversock+1, &rdfs, NULL, NULL, &tv) == 0 ) {
        set_errormsg("remote server %s read timeout", client->lastserver);
	/* FIXME
	 * writeclient(client,"500 Remote server read timeout");
	 */
        client->connected = 0;
	return 0;
    }

    bread = read(client->serversock, buf, size);
    if ( bread == -1 ) {
	set_errormsg("cant read from server %s %m", client->lastserver);
	return 0;
    }
    buf[bread] = 0;
    if ( cfg.LogReadserver )
        syslog(LOG_DEBUG,"readserver: %s", buf);
    client->serverbytes += (ulong)bread;
    return bread;
}


PROTO char * readserverline(char *buf, int size) {
    fd_set rdfs;
    struct timeval tv;
                                                                                                                           
    FD_ZERO(&rdfs);
    FD_SET(client->serversock, &rdfs);
                                                                                                                           
    tv.tv_sec  = cfg.ServerReadTimeout;
    tv.tv_usec = 0;
                                                                                                                           
    if ( select(client->serversock+1, &rdfs, NULL, NULL, &tv) == 0 ) {
        set_errormsg("remote server %s read timeout", client->lastserver);
	/* FIXME
	 * writeclient(client,"500 Remote server read timeout");
	 */
        client->connected = 0;
        return NULL;
    }
                                                                                                                           
    return fgetsrn(buf,size,client->serversock);
}


/* 
 * readclient() read a single line from the client
 */
PROTO char * readclient(CLIENT *client, char *buf, int size, int nowait) {
    fd_set rdfs;
    struct timeval tv;

    if ( ! nowait ) {
	FD_ZERO(&rdfs);
        FD_SET(client->socket, &rdfs);

        tv.tv_sec  = cfg.ClientReadTimeout;
        tv.tv_usec = 0;

	if ( select(client->socket+1, &rdfs, NULL, NULL, &tv) == 0 ) {
	    client->timeout++;
	    return NULL;
	}
    }
    return fgetsrn(buf,size,client->socket);
}

/* 
 * nreadclient() same as readclient but returns num bytes read
 */
PROTO int nreadclient(CLIENT *client, char *buf, int size, int nowait) {
    fd_set rdfs;
    struct timeval tv;

    if ( ! nowait ) {
	FD_ZERO(&rdfs);
        FD_SET(client->socket, &rdfs);

        tv.tv_sec  = cfg.ClientReadTimeout;
        tv.tv_usec = 0;

	if ( select(client->socket+1, &rdfs, NULL, NULL, &tv) == 0 ) {
	    client->timeout++;
	    return 0;
	}
    }
    return nfgetsrn(buf,size,client->socket);
}


/*
 * writeserver() write to the currently selected server
 */
PROTO int writeserver(CLIENT *client, char *str, ...) {
    va_list ap;
    size_t nc;
    char *s;
    
    va_start(ap, str);
    CLEARBUFS
    vsprintf(client->sbuf, str, ap);
    s = fixrn(client->sbuf);
    
    if ( cfg.LogWriteserver ) {
	syslog(LOG_DEBUG,"writeserver %s",s);
    }

    if ( (nc = write(client->serversock, s, strlen(s))) == -1 ) {
        syslog(LOG_ERR,"cant write to server %s %m", client->lastserver);
	return 0;
    }
    free(s);
    va_end(ap);
    return nc;
}


/*
 * writeclient() write to the client
 * return 0 on success, -1 on failure
 */
PROTO int writeclient(CLIENT *client, char *str, ...) {
    va_list ap;
    size_t nc;
    char *s;
    
    va_start(ap, str);
    CLEARBUFS
    vsprintf(client->sbuf, str, ap);
    s = fixrn(client->sbuf);
    va_end(ap);

    if ( cfg.LogWriteclient && atoi(s) > 199 )  {
	syslog(LOG_DEBUG,"writeclient %s",s);
    }

    if ( (nc = write(client->socket, s, strlen(s))) == -1 ) {
	set_errormsg("cant write to client %m");
	return -1;
    }
    free(s);
    return 0;
}


/*
 * swriteclient() statically write to the client
 * return 0 on success, -1 on failure
 * no vargs on this one
 */
PROTO int swriteclient(CLIENT *client, char *str) {

    char *s = fixrn(str);

    if ( cfg.LogWriteclient && atoi(s) > 199 )  {
	syslog(LOG_DEBUG,"swriteclient %s",s);
    }

    if ( (write(client->socket, s, strlen(s))) == -1 ) {
	set_errormsg("cant swrite to client %m");
	return -1;
    }
    free(s);
    return 0;
}


/*
 * slwriteclient() statically write to the client and ratelimit
 * return 0 on success, -1 on failure
 * no vargs on this one
 */
PROTO int slwriteclient(CLIENT *client, char *str, size_t size, int limit) {
    int sleeptime = 0;
    struct timeval sleep,end;
    int slept = 0;

    fd_set rdfs;
    struct timeval tv;

    if ( 1 ) {
	FD_ZERO(&rdfs);
        FD_SET(client->socket, &rdfs);

        tv.tv_sec  = cfg.ClientReadTimeout;
        tv.tv_usec = 0;

	if ( select(client->socket+1, NULL, &rdfs, NULL, &tv) == 0 ) {
	    set_errormsg("slwrite timeout");
	    return -1;
	}
    }

    if ( (write(client->socket, str, size)) == -1 ) {
	set_errormsg("cant slwrite to client %m");
	return -1;
    }

    /* silly rate limiter */

    if ( client->profile->MaxBytesPerSecond > 1  && limit == 1 ) {
	if ( client->rl_bytes == 0 ) gettimeofday(&client->rl_start,NULL);
        gettimeofday(&end,NULL);
        client->rl_bytes += size;

        if (end.tv_usec < client->rl_start.tv_usec) {
            end.tv_usec += 1000000;
            end.tv_sec -= 1;
        }
        sleep.tv_usec = end.tv_usec - client->rl_start.tv_usec;
        sleep.tv_sec  = end.tv_sec  - client->rl_start.tv_sec;

        sleeptime = (sleep.tv_sec * 1000000) + sleep.tv_usec; /* total usec elapsed */
        client->rl_curbps = (float)client->rl_bytes / (float)(sleeptime / 1000000);

        /* max 100mbps limiting, above is ignored */
        if ( client->profile->MaxBytesPerSecond < client->rl_curbps ) { 
            sleeptime = 1000000 - (1000000 / (client->rl_curbps / (float)client->profile->MaxBytesPerSecond ));
            usleep(sleeptime);
            slept += sleeptime;
        }
    }

    return 0;
}


/**
 **
 **  Server Connection Functions
 **
 **/


PROTO int connect_socket(char *server, int port) {
    struct sockaddr_in	name;
    struct hostent	*hostinfo;
    int sock;

    syslog(LOG_NOTICE,"connecting to server %s",server);

    sock = socket(PF_INET, SOCK_STREAM, 0);

    name.sin_family = AF_INET;
    if ( (hostinfo = gethostbyname(server)) == NULL ) {
	close(sock);
	return -1;
    }
 
    name.sin_addr = *((struct in_addr *)hostinfo->h_addr);
    name.sin_port = htons(port);

    if ( connect(sock, (struct sockaddr *) &name, sizeof(struct sockaddr)) == -1 ) {
	syslog(LOG_ERR,"connect to server %s failed (%m)", server);
	close(sock);
	return -1;
    }

    return sock;
}


PROTO char* handshake_nntp(int sock, char *buf, int size, int timeout) {
    int n;

    if ( (n=read_socket(sock, buf, size, timeout)) <= 0 )
	return("Connection Failed: %m");

    buf[n] = 0;

    if ( atoi(buf) >= 300 )
	return buf;

    return NULL;
}
		     

PROTO int connect_server_nntp(CLIENT *client, char *server, int port, int timeout) {
    int sock;
    
    if ( (sock=connect_socket(server, port)) == -1 )
	return 1;

    if ( handshake_nntp(sock, client->sbuf, cfg.BufSize, timeout) != NULL ) {
        syslog(LOG_ERR, "%s: error remote server %s response %s", client->hostname, server, client->sbuf);
        close(sock);
	return 1;
    }

    client->serversock = sock;
    client->connected=1;
    client->serverarts=0;
    client->serverbytes=0;
    strncpy(client->lastserver, server, MAX_HOSTNAME-1);
    return 0;
}


PROTO void send_quit(CLIENT *client) {
    if ( client->connected == 1 ) {
	if ( ! writeserver(client, "QUIT") ) return;
        if ( ! readserver(client, client->bbuf, cfg.BufSize) ) return;
    }
}

/*
 * disconnect_server() disconnect from currently selected backend news server
 */
PROTO int disconnect_server(CLIENT *client) {
    send_quit(client);
    client->groupserver = NULL;
    client->group = NULL;
    close(client->serversock);
    client->connected=0;
    syslog(LOG_DEBUG,"disconnected from server %s articles %d bytes %lu", client->lastserver, client->serverarts, client->serverbytes);
    return 0;
}

PROTO char* fgetsrn(char* ptr, int maxlen, int fd)
{
	int	n, rc;
	char	c;

	for (n = 1; n < maxlen; n++) {
		if ( (rc = read(fd, &c, 1)) == 1) {
			*ptr++ = c;
			if (c == '\n')
				break;
		} else if (rc == 0) {
			if (n == 1)
				return(NULL);	/* EOF, no data read */
			else
				break;		/* EOF, some data was read */
		} else {
			if ( errno ) set_errormsg("fgetsrn: %m");
			return(NULL);	/* error */
		}
	}

	*ptr = 0;
	return(ptr);
}

PROTO int nfgetsrn(char* ptr, int maxlen, int fd)
{
	int	n, rc;
	char	c;

	for (n = 1; n < maxlen; n++) {
		if ( (rc = read(fd, &c, 1)) == 1) {
			*ptr++ = c;
			if (c == '\n')
				break;
		} else if (rc == 0) {
			if (n == 1)
				return(0);	/* EOF, no data read */
			else
				break;		/* EOF, some data was read */
		} else {
			if ( errno ) set_errormsg("nfgetsrn: %m");
			return(0);	/* error */
		}
	}

	*ptr = 0;
	return(n);
}
